@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,10 +17,10 @@
 #
 """ Locating modules and package source on disk.
 
-The actual import of a module would already execute code that changes
-things. Imagine a module that does ``os.system()``, it would be done during
-compilation. People often connect to databases, and these kind of things,
-at import time.
+The actual import of a module would already execute code that changes things.
+Imagine a module that does ``os.system()``, it would be done during
+compilation. People often connect to databases, and these kind of things, at
+import time.
 
 Therefore CPython exhibits the interfaces in an ``imp`` module in standard
 library, which one can use those to know ahead of time, what file import would
@@ -30,16 +30,18 @@ exactly like CPython does, so we implement here a multi step search process
 that is compatible.
 
 This approach is much safer of course and there is no loss. To determine if
-it's from the standard library, one can abuse the attribute ``__file__`` of
-the ``os`` module like it's done in ``isStandardLibraryPath`` of this module.
+it's from the standard library, one can abuse the attribute ``__file__`` of the
+``os`` module like it's done in ``isStandardLibraryPath`` of this module.
 
 """
 
 from __future__ import print_function
 
+import hashlib
 import imp
 import os
 import sys
+import zipfile
 from logging import warning
 
 from nuitka import Options
@@ -47,7 +49,9 @@ from nuitka.containers.oset import OrderedSet
 from nuitka.importing import StandardLibrary
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
+from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.FileOperations import listDir
+from nuitka.utils.ModuleNames import ModuleName
 
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
 from .Whitelisting import isWhiteListedNotExistingModule
@@ -59,6 +63,7 @@ warned_about = set()
 # Directory where the main script lives. Should attempt to import from there.
 main_path = None
 
+
 def setMainScriptDirectory(main_dir):
     """ Initialize the main script directory.
 
@@ -78,26 +83,15 @@ def isPackageDir(dirname):
         extra packages provided via "*.pth" file tricks by "site.py" loading.
     """
 
-    return os.path.isdir(dirname) and \
-           (
-               python_version >= 300 or
-               os.path.isfile(os.path.join(dirname, "__init__.py")) or
-               isPreloadedPackagePath(dirname)
-           )
-
-
-def getPackageNameFromFullName(full_name):
-    if '.' in full_name:
-        return full_name[:full_name.rfind('.')]
-    else:
-        return None
-
-
-def getModuleFullNameFromPackageAndName(package, name):
-    if not package:
-        return name
-    else:
-        return package + '.' + name
+    return (
+        "." not in os.path.basename(dirname)
+        and os.path.isdir(dirname)
+        and (
+            python_version >= 300
+            or os.path.isfile(os.path.join(dirname, "__init__.py"))
+            or isPreloadedPackagePath(dirname)
+        )
+    )
 
 
 def getExtensionModuleSuffixes():
@@ -107,8 +101,22 @@ def getExtensionModuleSuffixes():
 
 
 def getModuleNameAndKindFromFilename(module_filename):
+    """ Given a filename, decide the module name and kind.
+
+    Args:
+        module_name - file path of the module
+    Returns:
+        Tuple with the name of the module basename, and the kind of the
+        module derived from the file suffix. Can be None, None if is is not a
+        known file suffix.
+    Notes:
+        This doesn't concern itself with packages, that needs to be tracked
+        by the using code. It cannot be decided from the filename at all.
+    """
+
+    # TODO: This does not handle ".pyw" files it seems.
     if os.path.isdir(module_filename):
-        module_name = os.path.basename(module_filename)
+        module_name = ModuleName(os.path.basename(module_filename))
         module_kind = "py"
     elif module_filename.endswith(".py"):
         module_name = os.path.basename(module_filename)[:-3]
@@ -119,7 +127,7 @@ def getModuleNameAndKindFromFilename(module_filename):
                 continue
 
             if module_filename.endswith(suffix):
-                module_name = os.path.basename(module_filename)[:-len(suffix)]
+                module_name = os.path.basename(module_filename)[: -len(suffix)]
                 module_kind = "shlib"
 
                 break
@@ -137,12 +145,13 @@ def isWhiteListedImport(node):
 
 
 def warnAbout(importing, module_name, parent_package, level, tried_names):
-    # This probably should not be dealt with here.
+    # This probably should not be dealt with here, pylint: disable=too-many-branches
     if module_name == "":
         return
 
-    if not isWhiteListedImport(importing) and \
-       not isWhiteListedNotExistingModule(module_name):
+    if not isWhiteListedImport(importing) and not isWhiteListedNotExistingModule(
+        module_name
+    ):
         key = module_name, parent_package, level
 
         if key not in warned_about:
@@ -165,22 +174,23 @@ def warnAbout(importing, module_name, parent_package, level, tried_names):
             else:
                 level_desc = "%d package levels up" % level
 
-            if parent_package is not None:
-                warning(
-                    "%s: Cannot find '%s' in package '%s' %s (tried %s).",
-                    importing.getSourceReference().getAsString(),
-                    module_name,
-                    parent_package,
-                    level_desc,
-                    ','.join(tried_names)
-                )
-            else:
-                warning(
-                    "%s: Cannot find '%s' %s.",
-                    importing.getSourceReference().getAsString(),
-                    module_name,
-                    level_desc
-                )
+            if _debug_module_finding:
+                if parent_package is not None:
+                    warning(
+                        "%s: Cannot find '%s' in package '%s' %s (tried %s).",
+                        importing.getSourceReference().getAsString(),
+                        module_name,
+                        parent_package,
+                        level_desc,
+                        ",".join(tried_names),
+                    )
+                else:
+                    warning(
+                        "%s: Cannot find '%s' %s.",
+                        importing.getSourceReference().getAsString(),
+                        module_name,
+                        level_desc,
+                    )
 
 
 def normalizePackageName(module_name):
@@ -188,7 +198,7 @@ def normalizePackageName(module_name):
     # platform, we either cannot look into it, or we require that we resolve it
     # here correctly.
     if module_name == "os.path":
-        module_name = os.path.basename(os.path.__name__)
+        module_name = ModuleName(os.path.basename(os.path.__name__))
 
     return module_name
 
@@ -199,25 +209,25 @@ def findModule(importing, module_name, parent_package, level, warn):
         The package name can be None of course. Level is the same
         as with "__import__" built-in. Warnings are optional.
 
-        Returns a triple of package name the module is in, filename of
-        it, which can be a directory for packages, and the location
-        method.
+        Returns:
+            Returns a triple of package name the module is in, filename of
+            it, which can be a directory for packages, and the location
+            method used.
     """
-
     # We have many branches here, because there are a lot of cases to try.
     # pylint: disable=too-many-branches
+
+    assert type(module_name) is ModuleName
+
     if _debug_module_finding:
         print(
-            "findModule: Enter to search %r in package %r level %s." % (
-                module_name,
-                parent_package,
-                level
-            )
+            "findModule: Enter to search %r in package %r level %s."
+            % (module_name, parent_package, level)
         )
 
     # Do not allow star imports to get here. We just won't find modules with
     # that name, but it would be wasteful.
-    assert module_name != '*'
+    assert module_name != "*"
 
     tried_names = []
 
@@ -225,7 +235,7 @@ def findModule(importing, module_name, parent_package, level, warn):
         # TODO: Should give a warning and return not found if the levels
         # exceed the package name.
         if parent_package is not None:
-            parent_package = '.'.join(parent_package.split('.')[:-level+1])
+            parent_package = ".".join(parent_package.split(".")[: -level + 1])
 
             if parent_package == "":
                 parent_package = None
@@ -234,73 +244,64 @@ def findModule(importing, module_name, parent_package, level, warn):
 
     # Try relative imports first if we have a parent package.
     if level != 0 and parent_package is not None:
-        full_name = normalizePackageName(parent_package + '.' + module_name)
-
-        if full_name.endswith('.'):
-            full_name = full_name[:-1]
+        if module_name:
+            full_name = ModuleName(parent_package + "." + module_name)
+        else:
+            full_name = ModuleName(parent_package)
 
+        full_name = normalizePackageName(full_name)
         tried_names.append(full_name)
 
         try:
-            module_filename = _findModule(
-                module_name = full_name,
-            )
+            module_filename = _findModule(module_name=full_name)
         except ImportError:
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
             if _debug_module_finding:
                 print(
-                    "findModule: Relative imported module '%s' as '%s' in filename '%s':" % (
-                        module_name,
-                        full_name,
-                        module_filename
-                    )
+                    "findModule: Relative imported module '%s' as '%s' in filename '%s':"
+                    % (module_name, full_name, module_filename)
                 )
 
-            return getPackageNameFromFullName(full_name), module_filename, "relative"
+            return full_name.getPackageName(), module_filename, "relative"
 
     if level <= 1 and module_name != "":
         module_name = normalizePackageName(module_name)
         tried_names.append(module_name)
 
-        package_name = getPackageNameFromFullName(module_name)
+        package_name = module_name.getPackageName()
 
         # Built-in module names must not be searched any further.
         if module_name in sys.builtin_module_names:
             if _debug_module_finding:
                 print(
-                    "findModule: Absolute imported module '%s' in as built-in':" % (
-                        module_name,
-                    )
+                    "findModule: Absolute imported module '%s' in as built-in':"
+                    % (module_name,)
                 )
             return package_name, None, "built-in"
 
         try:
-            module_filename = _findModule(
-                module_name = module_name,
-            )
+            module_filename = _findModule(module_name=module_name)
         except ImportError:
             # For relative import, that is OK, we will still try absolute.
             pass
         else:
             if _debug_module_finding:
                 print(
-                    "findModule: Found absolute imported module '%s' in filename '%s':" % (
-                        module_name,
-                        module_filename
-                    )
+                    "findModule: Found absolute imported module '%s' in filename '%s':"
+                    % (module_name, module_filename)
                 )
 
             return package_name, module_filename, "absolute"
 
     if warn:
         warnAbout(
-            importing      = importing,
-            module_name    = module_name,
-            parent_package = parent_package,
-            tried_names    = tried_names,
-            level          = level
+            importing=importing,
+            module_name=module_name,
+            parent_package=parent_package,
+            tried_names=tried_names,
+            level=level,
         )
 
     return None, None, "not-found"
@@ -309,6 +310,7 @@ def findModule(importing, module_name, parent_package, level, warn):
 # Some platforms are case insensitive.
 case_sensitive = not sys.platform.startswith(("win", "cygwin", "darwin"))
 
+
 def _findModuleInPath2(module_name, search_path):
     """ This is out own module finding low level implementation.
 
@@ -346,23 +348,17 @@ def _findModuleInPath2(module_name, search_path):
                 file_path = os.path.join(package_directory, package_file_name)
 
                 if os.path.isfile(file_path):
-                    candidates.add(
-                        (entry, 1, package_directory)
-                    )
+                    candidates.add((entry, 1, package_directory))
                     break
             else:
                 if python_version >= 300:
-                    candidates.add(
-                        (entry, 2, package_directory)
-                    )
+                    candidates.add((entry, 2, package_directory))
 
         # Then, check out suffixes of all kinds.
         for suffix, _mode, _type in imp.get_suffixes():
             file_path = os.path.join(entry, module_name + suffix)
             if os.path.isfile(file_path):
-                candidates.add(
-                    (entry, 1, file_path)
-                )
+                candidates.add((entry, 1, file_path))
                 break
 
     if _debug_module_finding:
@@ -372,12 +368,7 @@ def _findModuleInPath2(module_name, search_path):
         # Ignore lower priority matches from package directories without
         # "__init__.py" file.
         min_prio = min(candidate[1] for candidate in candidates)
-        candidates = [
-            candidate
-            for candidate in
-            candidates
-            if candidate[1] == min_prio
-        ]
+        candidates = [candidate for candidate in candidates if candidate[1] == min_prio]
 
         # On case sensitive systems, no resolution needed.
         if case_sensitive:
@@ -395,20 +386,44 @@ def _findModuleInPath2(module_name, search_path):
     raise ImportError
 
 
+_egg_files = {}
+
+
+def _unpackPathElement(path_entry):
+    if not path_entry:
+        return "."  # empty means current directory
+
+    if os.path.isfile(path_entry) and path_entry.lower().endswith(".egg"):
+        if path_entry not in _egg_files:
+            with open(path_entry, "rb") as f:
+                checksum = hashlib.md5(f.read()).hexdigest()
+
+            target_dir = os.path.join(getCacheDir(), "egg-content", checksum)
+
+            zip_ref = zipfile.ZipFile(path_entry, "r")
+            zip_ref.extractall(target_dir)
+            zip_ref.close()
+
+            _egg_files[path_entry] = target_dir
+
+        return _egg_files[path_entry]
+
+    return path_entry
+
+
 def getPackageSearchPath(package_name):
     assert main_path is not None
 
     if package_name is None:
-        return [os.getcwd(), main_path] + sys.path
-    elif '.' in package_name:
-        parent_package_name, child_package_name = package_name.rsplit('.', 1)
+        return [os.getcwd(), main_path] + [
+            _unpackPathElement(path_element) for path_element in sys.path
+        ]
+    elif "." in package_name:
+        parent_package_name, child_package_name = package_name.rsplit(".", 1)
 
         result = []
         for element in getPackageSearchPath(parent_package_name):
-            package_dir = os.path.join(
-                element,
-                child_package_name
-            )
+            package_dir = os.path.join(element, child_package_name)
 
             if isPackageDir(package_dir):
                 result.append(package_dir)
@@ -442,7 +457,9 @@ def getPackageSearchPath(package_name):
         return result
 
 
-def _findModuleInPath(module_name, package_name):
+def _findModuleInPath(module_name):
+    package_name, module_name = module_name.splitModuleBasename()
+
     if _debug_module_finding:
         print("_findModuleInPath: Enter", module_name, "in", package_name)
 
@@ -466,16 +483,13 @@ def _findModuleInPath(module_name, package_name):
 
     try:
         module_filename = _findModuleInPath2(
-            module_name = module_name,
-            search_path = search_path
+            module_name=module_name, search_path=search_path
         )
     except SyntaxError:
         # Warn user, as this is kind of unusual.
         warning(
             "%s: Module cannot be imported due to syntax errors.",
-            module_name
-              if package_name is None else
-            package_name + '.' + module_name,
+            module_name if package_name is None else package_name + "." + module_name,
         )
 
         return None
@@ -485,17 +499,15 @@ def _findModuleInPath(module_name, package_name):
 
     return module_filename
 
+
 module_search_cache = {}
 
+
 def _findModule(module_name):
     if _debug_module_finding:
-        print(
-            "_findModule: Enter to search '%s'." % (
-                module_name,
-            )
-        )
+        print("_findModule: Enter to search '%s'." % (module_name,))
 
-    assert not module_name.endswith('.'), module_name
+    assert not module_name.endswith("."), module_name
 
     key = module_name
 
@@ -507,8 +519,8 @@ def _findModule(module_name):
 
         if result is ImportError:
             raise ImportError
-        else:
-            return result
+
+        return result
 
     try:
         module_search_cache[key] = _findModule2(module_name)
@@ -518,8 +530,8 @@ def _findModule(module_name):
         if new_module_name is None:
             module_search_cache[key] = ImportError
             raise
-        else:
-            module_search_cache[key] = _findModule(new_module_name)
+
+        module_search_cache[key] = _findModule(new_module_name)
 
     return module_search_cache[key]
 
@@ -528,18 +540,9 @@ def _findModule2(module_name):
     # Need a real module name.
     assert module_name != ""
 
-    if '.' in module_name:
-        package_part = module_name[ : module_name.rfind('.') ]
-        module_name = module_name[ module_name.rfind('.') + 1 : ]
-    else:
-        package_part = None
-
     preloaded_path = getPreloadedPackagePath(module_name)
 
     if preloaded_path is not None:
         return preloaded_path[0]
 
-    return _findModuleInPath(
-        module_name  = module_name,
-        package_name = package_part
-    )
+    return _findModuleInPath(module_name=module_name)
