@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,6 +19,8 @@
 
 """
 
+import __future__
+
 import ast
 from logging import warning
 
@@ -31,7 +33,10 @@ from nuitka.nodes.ContainerMakingNodes import (
     ExpressionMakeSetLiteral,
     ExpressionMakeTuple,
 )
-from nuitka.nodes.DictionaryNodes import ExpressionKeyValuePair, ExpressionMakeDict
+from nuitka.nodes.DictionaryNodes import (
+    ExpressionKeyValuePair,
+    ExpressionMakeDict,
+)
 from nuitka.nodes.ExceptionNodes import StatementReraiseException
 from nuitka.nodes.FrameNodes import (
     StatementsFrameAsyncgen,
@@ -44,7 +49,10 @@ from nuitka.nodes.ImportNodes import ExpressionBuiltinImport
 from nuitka.nodes.NodeBases import NodeBase
 from nuitka.nodes.NodeMakingHelpers import mergeStatements
 from nuitka.nodes.StatementNodes import StatementsSequence
-from nuitka.PythonVersions import needsSetLiteralReverseInsertion, python_version
+from nuitka.PythonVersions import (
+    needsSetLiteralReverseInsertion,
+    python_version,
+)
 
 
 def dump(node):
@@ -116,21 +124,15 @@ def detectFunctionBodyKind(nodes, start_value=None):
 
         indications.clear()
         indications.update(old)
-        del old
 
     def _check(node):
         node_class = node.__class__
 
         if node_class is ast.Yield:
             indications.add("Generator")
-        elif (
-            python_version >= 300 and node_class is ast.YieldFrom  # @UndefinedVariable
-        ):
+        elif python_version >= 300 and node_class is ast.YieldFrom:
             indications.add("Generator")
-        elif python_version >= 350 and node_class in (
-            ast.Await,  # @UndefinedVariable
-            ast.AsyncWith,  # @UndefinedVariable
-        ):
+        elif python_version >= 350 and node_class in (ast.Await, ast.AsyncWith):
             indications.add("Coroutine")
 
         # Recurse to children, but do not cross scope boundary doing so.
@@ -150,8 +152,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                 else:
                     assert False, (name, field, ast.dump(node))
         elif node_class in (ast.FunctionDef, ast.Lambda) or (
-            python_version >= 350
-            and node_class is ast.AsyncFunctionDef  # @UndefinedVariable
+            python_version >= 350 and node_class is ast.AsyncFunctionDef
         ):
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body"):
@@ -198,7 +199,8 @@ def detectFunctionBodyKind(nodes, start_value=None):
                             if gen.is_async:
                                 indications.add("Coroutine")
                                 break
-                            elif _checkCoroutine(gen):
+
+                            if _checkCoroutine(gen):
                                 break
                 else:
                     assert False, (name, field, ast.dump(node))
@@ -212,7 +214,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     _check(field)
                 else:
                     assert False, (name, field, ast.dump(node))
-        elif python_version >= 270 and node_class is ast.SetComp:  # @UndefinedVariable
+        elif python_version >= 270 and node_class is ast.SetComp:
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body", "comparators", "elt"):
                     pass
@@ -220,7 +222,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     _check(field[0].iter)
                 else:
                     assert False, (name, field, ast.dump(node))
-        elif python_version >= 270 and node_class is ast.DictComp:  # @UndefinedVariable
+        elif python_version >= 270 and node_class is ast.DictComp:
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body", "comparators", "key", "value"):
                     pass
@@ -228,10 +230,26 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     _check(field[0].iter)
                 else:
                     assert False, (name, field, ast.dump(node))
+        elif python_version >= 370 and node_class is ast.comprehension:
+            for name, field in ast.iter_fields(node):
+                if name in ("name", "target"):
+                    pass
+                elif name == "iter":
+                    # Top level comprehension iterators do not influence those.
+                    if node not in nodes:
+                        _check(field)
+                elif name == "ifs":
+                    for child in field:
+                        _check(child)
+                elif name == "is_async":
+                    if field:
+                        indications.add("Coroutine")
+                else:
+                    assert False, (name, field, ast.dump(node))
         elif node_class is ast.Name:
             if python_version >= 300 and node.id == "super":
                 flags.add("has_super")
-        elif python_version < 300 and node_class is ast.Exec:  # @UndefinedVariable
+        elif python_version < 300 and node_class is ast.Exec:
             flags.add("has_exec")
 
             if node.globals is None:
@@ -290,8 +308,6 @@ def buildNode(provider, node, source_ref, allow_none=False):
 
         if hasattr(node, "lineno"):
             source_ref = source_ref.atLineNumber(node.lineno)
-        else:
-            source_ref = source_ref
 
         if kind in build_nodes_args3:
             result = build_nodes_args3[kind](
@@ -361,13 +377,24 @@ def buildAnnotationNode(provider, node, source_ref):
 
         _host_node.body[0].annotation = node
 
-        r = compile(_host_node, "<annotations>", "exec", 1048576, dont_inherit=True)
+        r = compile(
+            _host_node,
+            "<annotations>",
+            "exec",
+            __future__.CO_FUTURE_ANNOTATIONS,
+            dont_inherit=True,
+        )
 
         # Using exec here, to compile the ast node tree back to string,
         # there is no accessible "ast.unparse", and this works as a hack
         # to convert our node to a string annotation, pylint: disable=exec-used
         m = {}
-        exec(r, m)  # @UndefinedVariable
+        exec(r, m)
+
+        # TODO: In Python3.9, we should get https://bugs.python.org/issue35143
+        # to be used.
+        if Options.is_debug:
+            assert python_version <= 390
 
         return makeConstantRefNode(
             constant=m["__annotations__"]["x"], source_ref=source_ref
